/// @nodoc
library assets_indexer;

import 'dart:async';
import 'dart:io';

import 'package:assets_indexer/src/font_families_builder.dart';
import 'package:build/build.dart';
import 'package:assets_indexer/src/assets_builder.dart';
import 'package:quartet/quartet.dart';
import 'package:yaml/yaml.dart';
import 'src/utils.dart';

/// @nodoc
class EmptyBuilder implements Builder {
  @override
  Map<String, List<String>> get buildExtensions => {};

  @override
  FutureOr<void> build(BuildStep buildStep) {}
}

/// @nodoc
Builder assetsIndexer(BuilderOptions options) {
  var builder = AssetsBuilder(options);
  preExecuter(builder);
  return EmptyBuilder();
}

/// @nodoc
Builder fontFamiliesIndexer(BuilderOptions options) {
  var builder = FontFamiliesBuilder(options);
  builder.build(null);
  return EmptyBuilder();
}

/// @nodoc
preExecuter(AssetsBuilder builder) {
  Directory directory = Directory(outDirectory);
  directory.createSync(recursive: true);
  (() async {
    var assetDirs = [];
    File pubSpec = File("./pubspec.yaml");
    Map<String, List<String>> pacakgePathsToReplace = {};
    var pubSpecTextContent = await pubSpec.readAsString();
    Map yaml = loadYaml(pubSpecTextContent);
    if (yaml['flutter']['assets'] != null &&
        yaml['flutter']['assets'].isNotEmpty) {
      for (String assetPath in yaml['flutter']['assets']) {
        if (assetPath.endsWith("/") || assetPath.endsWith("\\")) {
          assetPath = assetPath.replaceRange(
              assetPath.length - 1, assetPath.length, '');
        }

        var originalPath = assetPath;
        StringModifier? pathModifier;

        if (assetPath.startsWith('packages/')) {
          var splitAssetPath = assetPath.split('/');
          if (splitAssetPath.last.contains(".")) {
            splitAssetPath.removeRange(
                splitAssetPath.length - 2, splitAssetPath.length);
          }
          var packageName = splitAssetPath[1];
          splitAssetPath.removeRange(0, 2);
          var packageAssetPath = "lib/${splitAssetPath.join('/')}";
          originalPath = packageAssetPath;
          pathModifier = (filePath) {
            var finalFilePath =
                'packages/$packageName/${filePath.split('lib/').last}';
            if (!pacakgePathsToReplace.containsKey(assetPath)) {
              pacakgePathsToReplace[assetPath] = [];
            }
            pacakgePathsToReplace[assetPath]?.add(finalFilePath);
            return finalFilePath;
          };
        }

        if (!originalPath.contains(".") &&
            (await Directory(originalPath).exists())) {
          assetDirs.add(assetPath);
          for (var inpDirFile in Directory(originalPath).listSync()) {
            var ext = inpDirFile.path.split('.').last;
            if (builder.buildExtensions.keys.toList().contains(".$ext")) {
              await builder.buildAsset(
                  inpDirFile.path, assetPath.split('/').last,
                  pathModifier: pathModifier);
            }
          }
        }
      }
    }
    if (pacakgePathsToReplace.isNotEmpty) {
      var outContent = [];
      for (var packagePathEntry in pacakgePathsToReplace.entries) {
        var finalFilePaths =
            packagePathEntry.value.map((e) => '\t\t- $e').join('\n');
        outContent.add(finalFilePaths);
      }
      //pubSpec.writeAsStringSync(pubSpecTextContent);
      print(outContent.join('\n'));
    }

    assetDirs = assetDirs.map((e) => snakeCase(e.split("/").last)).toList();
    if (Platform.isWindows) return;
    for (var file in directory.listSync()) {
      if (file.existsSync() &&
          file.path.split("/").last != "app_fonts.asset.dart") {
        var fileDir = file.path.split("/").last.split(".").first;
        if (!assetDirs.contains(fileDir)) {
          log.info(
              "Removed : ${file.path} since its directory removed from assets paths");
          file.deleteSync();
        }
      }
    }
  })();
}
